import ssl
import httpx
from enum import Enum
from typing import List, TypeVar, Union
from dataclasses import dataclass, field

Self = TypeVar("Self", bound="Route")

# The constant includes all necessary error messages that can occurs, if you establish a connection to the Grafana API.
ERROR_MESSAGES: list = ["invalid API key", "Invalid API key", "Expired API key"]


# TODO SSO Integrationtests
# TODO Check the Alerting case https://github.com/grafana/grafana/blob/main/pkg/services/ngalert/api/tooling/post.json


class APIEndpoints(Enum):
    """The class includes all necessary API endpoint strings to connect the Grafana API"""

    api_prefix: str = "/api"
    version_1: str = "v1"

    SEARCH: str = f"{api_prefix}/search"
    DASHBOARDS: str = f"{api_prefix}/dashboards"
    FOLDERS: str = f"{api_prefix}/folders"
    LEGACY_ALERTS: str = f"{api_prefix}/alerts"
    ALERT_NOTIFICATIONS: str = f"{api_prefix}/alert-notifications"
    ALERTS_ALERTMANAGER: str = f"{api_prefix}/alertmanager"
    ALERTS_PROMETHEUS: str = f"{api_prefix}/prometheus"
    ALERTS_RULER: str = f"{api_prefix}/ruler"
    ALERTS_TESTING: str = f"{api_prefix}/{version_1}"
    ALERTS_NGALERT: str = f"{api_prefix}/{version_1}/ngalert"
    ALERTING_PROVISIONING: str = f"{api_prefix}/{version_1}/provisioning"
    DATASOURCES: str = f"{api_prefix}/datasources"
    DATASOURCE_QUERY: str = f"{api_prefix}/tsdb/query"
    DATASOURCE_PERMISSIONS: str = f"{api_prefix}/access-control/datasources"
    SHORT_URLS: str = f"{api_prefix}/short-urls"
    ORGANISATION: str = f"{api_prefix}/org"
    ORGANISATIONS: str = f"{api_prefix}/orgs"
    USER: str = f"{api_prefix}/user"
    USERS: str = f"{api_prefix}/users"
    SNAPSHOTS: str = f"{api_prefix}/snapshots"
    DASHBOARD_SNAPSHOTS: str = f"{api_prefix}/dashboard/snapshots"
    PLAYLISTS: str = f"{api_prefix}/playlists"
    TEAMS: str = f"{api_prefix}/teams"
    QUERY_HISTORY: str = f"{api_prefix}/query-history"
    REPORTING: str = f"{api_prefix}/reports"
    LICENSING: str = f"{api_prefix}/licensing"
    FRONTEND: str = f"{api_prefix}/frontend"
    LOGIN: str = f"{api_prefix}/login"
    AUTHENTICATION: str = f"{api_prefix}/auth/keys"
    EXTERNAL_GROUPS: str = f"{api_prefix}/teams"
    USER_PREFERENCES: str = f"{api_prefix}/user/preferences"
    ORG_PREFERENCES: str = f"{api_prefix}/org/preferences"
    ANNOTATIONS: str = f"{api_prefix}/annotations"
    ADMIN: str = f"{api_prefix}/admin"
    SERVICE_ACCOUNTS: str = f"{api_prefix}/serviceaccounts"
    RBAC: str = f"{api_prefix}/access-control"
    LIBRARY: str = f"{api_prefix}/library-elements"
    SSO_SETTINGS: str = f"{api_prefix}/{version_1}/sso-settings"


class RequestsMethods(Enum):
    """The class includes all necessary method values to establish an HTTP/ HTTPS connection to the Grafana API endpoints"""

    GET: str = "GET"
    PUT: str = "PUT"
    POST: str = "POST"
    PATCH: str = "PATCH"
    DELETE: str = "DELETE"


class SortDirection(Enum):
    ASC = "alpha-asc"
    DESC = "alpha-desc"


@dataclass
class APIModel:
    """The class includes all necessary variables to establish a connection to the Grafana API endpoints

    Args:
        host (str): Specify the host of the Grafana system
        token (str): Specify the access token of the Grafana system
        username (str): Specify the username of the Grafana system
        password (str): Specify the password of the Grafana system
        timeout (float): Specify the timeout of the Grafana system
        headers (dict): Specify the headers of the Grafana system
        http2_support (bool): Specify if you want to use HTTP/2
        ssl_context (ssl.SSLContext): Specify the custom ssl context of the Grafana system
        num_pools (int): Specify the number of the connection pool
        retries (any): Specify the number of the retries. Please use False as parameter to disable the retries
    """

    host: str
    token: str = None
    username: str = None
    password: str = None
    headers: dict = None
    timeout: float = 10.0
    http2_support: bool = False
    ssl_context: ssl.SSLContext = httpx.create_ssl_context()
    num_pools: int = 10
    retries: any = 10


@dataclass
class DatasourceQuery:
    """The class includes all necessary variables to specify a query for the datasource search endpoint

    Args:
        datasource_id (int): Specify the id of the data source
        raw_sql (str): Specify the raw SQL string to search inside the Grafana system
        ref_id (str): Specify a reference id of the search command (default A)
        interval_ms (int): Specify the time interval in milliseconds of output format (default 1000)
        max_data_points (int): Specify maximum amount of data points that dashboard panel can render (default 100)
        output_format (str): Specify the output format of the query (default time_series)
    """

    datasource_id: int
    raw_sql: str
    ref_id: str = "A"
    interval_ms: int = 1000
    max_data_points: int = 100
    output_format: str = "time_series"


@dataclass
class DatasourceRuleQuery:
    """The class includes all necessary variables to specify a query for the datasource rule search endpoint

    Args:
        datasource_uid (str): Specify the uid of the data source
        model (dict): Specify the model of the search command
        query_type (str): Specify the query time of the search command
        ref_id (str): Specify a reference id of the search command
        relative_time_range (dict): Specify the related time range of the search command
    """

    datasource_uid: str
    model: dict
    query_type: str
    ref_id: str
    relative_time_range: dict


@dataclass
class DatasourcePermission:
    """The class includes the necessary variables to generate a datasource permission object that is necessary to communicate with the Grafana datasource permissions endpoint

    Args:
        permission (Union[str, None]): Specify the datasource permission. Can be query, edit, admin or None. To remove a permission, set the permission value to None

    Raises:
        ValueError: Missed specifying a necessary value

    Returns:
        permission (str): Returns the datasource permission
    """

    def __init__(self, permission: Union[str, None]):
        if permission is None:
            self.permission: str = ""
        elif permission.lower() == "query":
            self.permission: str = "Query"
        elif permission.lower() == "edit":
            self.permission: str = "Edit"
        elif permission.lower() == "admin":
            self.permission: str = "Admin"
        else:
            raise ValueError


@dataclass
class Alert:
    """The class includes all necessary variables to generate an alert object that is necessary to communicate with the Grafana alert endpoint

    Args:
        starts_at (str): Specify the start date of the alert
        ends_at (str): Specify end date of the alert
        annotations (dict): Specify the annotations of the alert
        generator_url (str):  Specify the url of the generator endpoint
        labels (dict): Specify labels of the alert
    """

    starts_at: str
    ends_at: str
    annotations: dict
    generator_url: str
    labels: dict


@dataclass
class AlertRuleQueryModelCondition:
    """The class includes all necessary variables to generate an alert rule query model condition object that is necessary to communicate with the Grafana alert provisioning endpoint
    Args:
        evaluator_params (list): Specify the evaluator parameters
        evaluator_type (str): Specify the evaluator type
        operator_type (str): Specify the operator type
        query_params (list): Specify the query parameters
        reducer_params (list): Specify the reducer parameters
        reducer_type (str): Specify the reducer type
        type (str): Specify the type
    """

    evaluator_params: list
    evaluator_type: str
    operator_type: str
    query_params: list
    reducer_params: list
    reducer_type: str
    type: str


@dataclass
class AlertRuleQueryModel:
    """The class includes all necessary variables to generate an alert rule query model object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        conditions (list): Specify the conditions list based on the AlertRuleQueryModelCondition objects
        datasource (dict): Specify the datasource dictionary
        expression (str): Specify the expression string
        hide (bool): Specify the if the query should be hided
        interval_ms (int): Specify the interval in ms
        max_data_points (int): Specify the max data points
        ref_id (str): Specify the unique identifier of the alert rule query model
        type (str): Specify the corresponding type
    """

    conditions: List[AlertRuleQueryModelCondition]
    datasource: dict
    expression: str
    hide: bool
    interval_ms: int
    max_data_points: int
    ref_id: str
    type: str


@dataclass
class AlertQuery:
    """The class includes all necessary variables to generate an alert query object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        datasource_uid (str): Specify the datasource uid
        model (AlertRuleQueryModel): Specify the model as AlertRuleQueryModel
        query_type (str): Specify the query type
        ref_id (str): Specify the unique identifier of the alert query
        relative_time_range_from (int): Specify the relative from time range
        relative_time_range_to (int): Specify the relative to time range
    """

    datasource_uid: str
    model: AlertRuleQueryModel
    query_type: str
    ref_id: str
    relative_time_range_from: int
    relative_time_range_to: int


@dataclass
class AlertRule:
    """The class includes all necessary variables to generate an alert rule object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        condition (str): Specify the condition
        data (list): Specify the data as AlertQuery list
        exec_err_state (str): Specify the execution error state
        folder_uid (str): Specify the folder uid
        no_data_state (str): Specify the no data state
        org_id (int): Specify the org id
        rule_group (str):  Specify the rule group of the alert rule
        title (str): Specify the title of the alert rule
        uid (str): Specify the uid of the alert rule
        for_time (int): Specify the for duration as integer
        annotations (dict): Specify the annotations dictionary (default None)
        updated (str): Specify the updated date time as string (default None)
        provenance (str): Specify the provenance (default None)
        id (int): Specify the alert rule id (default 0)
        labels (dict): Specify the labels as dictionary (default None)
    """

    condition: str
    data: List[AlertQuery]
    exec_err_state: str
    folder_uid: str
    no_data_state: str
    org_id: int
    rule_group: str
    title: str
    uid: str
    for_time: str
    annotations: dict = None
    updated: str = None
    provenance: str = None
    id: int = 0
    labels: dict = None


@dataclass
class EmbeddedContactPoint:
    """The class includes all necessary variables to generate an embedded contact point object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        name (str): Specify the name of the embedded contact point
        type (str): Specify the type of the embedded contact point
        settings (dict): Specify the settings of the embedded contact point
        disable_resolve_message (bool): Specify if the resolve message should be disabled (default None)
        provenance (str): Specify the provenance (default None)
        uid (str): Specify the uid of the embedded contact point (default None)
    """

    name: str
    type: str
    settings: dict
    disable_resolve_message: bool = None
    provenance: str = None
    uid: str = None


class MatchType(Enum):
    """The class includes all necessary values to set the corresponding match type"""

    MatchEqual: int = 0
    MatchNotEqual: int = 1
    MatchRegexp: int = 2
    MatchNotRegexp: int = 3


@dataclass
class Matcher:
    """The class includes all necessary variables to generate an alert rule route matcher object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        name (str): Specify the name of the matcher
        type (MatchType): Specify the type of the matcher
        value (str): Specify the value of the matcher
    """

    name: str
    type: MatchType
    value: str


@dataclass
class Route:
    """The class includes all necessary variables to generate an alert rule route that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        continue_parameter (bool): Specify the continue parameter
        group_by_str (List[str]): Specify the list of group by strings
        receiver (str): Specify the receiver
        provenance (str): Specify the provenance
        object_matchers (List[Matcher]):  Specify the list of object matchers (default None)
        group_interval (str): Specify the group time interval (default None)
        group_wait (str): Specify the group wait time (default None)
        repeat_interval (str): Specify the repeat interval (default None)
        routes (List[Route]): Specify the list of routes (default None)
        mute_time_intervals (List[str]): Specify the mute time interval as list (default None)
    """

    continue_parameter: bool
    group_by_str: List[str]
    receiver: str
    provenance: str
    object_matchers: List[Matcher] = None
    group_interval: str = None
    group_wait: str = None
    repeat_interval: str = None
    routes: List[Self] = None
    mute_time_intervals: List[str] = None


@dataclass
class TimeRange:
    """The class includes all necessary variables to generate a time range object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        start_time (str): Specify the start time e.g. 14:00
        start_time (str): Specify the end time e.g. 15:00
    """

    start_time: str
    end_time: str


@dataclass
class TimeInterval:
    """The class includes all necessary variables to generate a time interval object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        days_of_month (List[str]): Specify the days of month list (default None)
        months (List[str]):  Specify the months list (default None)
        times (TimeRange):  Specify the times list (default None)
        weekdays (List[str]):  Specify the weekdays list (default None)
        years (List[str]):  Specify the year range list (default None)
    """

    days_of_month: List[str] = None
    months: List[str] = None
    times: List[TimeRange] = None
    weekdays: List[str] = None
    years: List[str] = None


@dataclass
class MuteTimeInterval:
    """The class includes all necessary variables to generate a mute time interval object that is necessary to communicate with the Grafana alert provisioning endpoint

    Args:
        name (str): Specify the name of the mute time interval.
        time_intervals (List[TimeInterval]): Specify the list of time interval objects
    """

    name: str = None
    time_intervals: List[TimeInterval] = None


@dataclass
class Silence:
    """The class includes all necessary variables to generate a silence object that is necessary to communicate with the Grafana silence endpoint

    Args:
        starts_at (str): Specify the start date of the silence
        created_by (str): Specify the name of the silence creator
        ends_at (str): Specify end date of the silence
        comment (str):  Specify a custom comment for the silence
        id (str): Specify an id for the silence
        matchers (dict): Specify matchers for the silence
    """

    starts_at: str
    created_by: str
    ends_at: str
    comment: str
    id: str
    matchers: dict


@dataclass
class AlertmanagerConfig:
    """The class includes all necessary variables to generate an Alertmanager config object that is necessary to communicate and set up the Grafana Alertmanager endpoint

    Args:
        global_config (dict): Specify the global config of the Alertmanager
        inhibit_rules (list): Specify the inhibit rules of the Alertmanager
        mute_time_intervals (list): Specify the mute time intervals of the Alertmanager
        receivers (list):  Specify the receiver's of the Alertmanager
        route (dict): Specify the route of the Alertmanager
        templates (list): Specify an Alertmanager template
    """

    global_config: dict
    inhibit_rules: list
    mute_time_intervals: list
    receivers: list
    route: dict
    templates: list


@dataclass
class AlertmanagerReceivers:
    """The class includes all necessary variables to generate an Alertmanager receiver's object that is necessary to communicate and set up the Grafana Alertmanager receivers endpoint

    Args:
        name (str): Specify the name of the receiver
        email_configs (list): Specify the email configuration of the receiver's
        grafana_managed_receiver_configs (list): Specify the Grafana managed receiver configuration of the receiver's
        opsgenie_configs (list):  Specify the ops genie configuration of the receiver's
        pagerduty_configs (dict): Specify the pager duty configuration of the receiver's
        pushover_configs (list): Specify the push over configuration of the receiver's
        slack_configs (list): Specify the Slack configuration of the receiver's
        victorops_configs (list): Specify the victor ops configuration of the receiver's
        webhook_configs (list): Specify the webhook configuration of the receiver's
        wechat_configs (list): Specify the wechaty configuration of the receiver's
    """

    name: str
    email_configs: list
    grafana_managed_receiver_configs: list
    opsgenie_configs: list
    pagerduty_configs: list
    pushover_configs: list
    slack_configs: list
    sns_configs: list
    victorops_configs: list
    webhook_configs: list
    wechat_configs: list


@dataclass
class RulerRule:
    """The class includes all necessary variables to generate a Ruler rule object that is necessary to communicate and set up a Grafana Ruler rule

    Args:
        alert (str): Specify the name of the rule
        annotations (dict): Specify the annotations of the rule
        expr (str): Specify the expression of the rule
        grafana_alert (dict):  Specify the Grafana alert of the rule
        labels (dict): Specify labels of the rule
        record (str): Specify recode value of the rule
        for_id (int): Specify the id of the rule if you update an existing rule (default 0)
    """

    alert: str
    annotations: dict
    expr: str
    grafana_alert: dict
    labels: dict
    record: str
    for_id: int = 0


@dataclass
class UserObject:
    """The class includes all necessary variables to generate a User object that is necessary to update a Grafana User

    Args:
        email (str): Specify the name of the rule
        name (str): Specify the annotations of the rule
        login (str): Specify the expression of the rule
        theme (str):  Specify the Grafana alert of the rule
    """

    email: str
    name: str
    login: str
    theme: str


@dataclass
class PlaylistObject:
    """The class includes all necessary variables to generate a playlist object

    Args:
        name (str): Specify the name of the playlist
        interval (str): Specify the interval of the playlist
        items (list): Specify a list of PlaylistItemObjects
    """

    name: str
    interval: str
    items: list


@dataclass
class PlaylistItemObject:
    """The class includes all necessary variables to generate a playlist item object that is necessary to update a playlist

    Args:
        type (str): Specify the type of the playlist item
        value (str): Specify the value of the playlist item
        order (int): Specify the order of the playlist item
        title (str): Specify the title of the playlist item
    """

    type: str
    value: str
    order: int
    title: str


@dataclass
class TeamObject:
    """The class includes all necessary variables to generate a team object that is necessary to create a team

    Args:
        name (str): Specify the name of the team
        email (str): Specify the email of the team
        org_id (int): Specify the org_id of the team
    """

    name: str
    email: str
    org_id: int


@dataclass
class QueryDatasourceObject:
    """The class includes all necessary variables to generate a query datasource object that is necessary to create a query history object

    Args:
        type (str): Specify the type of the datasource query
        uid (str): Specify the uid of the datasource query
    """

    type: str
    uid: str


@dataclass
class QueryObject:
    """The class includes all necessary variables to generate a query object that is necessary to create a query history

    Args:
        ref_id (str): Specify the ref_id of the query history
        key (str): Specify the key of the query history
        scenario_id (str): Specify the scenario_id of the query history
        datasource (QueryDatasourceObject): Specify the datasource of the type QueryDatasourceObject
    """

    ref_id: str
    key: str
    scenario_id: str
    datasource: QueryDatasourceObject


@dataclass
class CorrelationObject:
    """The class includes all necessary variables to generate a find annotation object

    Args:
        source_datasource_uid (str): Specify the source datasource uid (default None)
        target_datasource_uid (str): Specify the target datasource uid (default None)
        label (str): Specify the label (default 100)
        description (str): Specify the description (default None)
        config_type (str): Specify the config type (default None)
        config_field (str): Specify the config field (default None)
        config_target (str): Specify the config target (default None)
    """

    source_datasource_uid: str = None
    target_datasource_uid: str = None
    label: str = None
    description: str = None
    config_type: str = None
    config_field: str = None
    config_target: dict = field(default_factory=dict)


@dataclass
class FindAnnotationObject:
    """The class includes all necessary variables to generate a find annotation object

    Args:
        from_value (int): Specify the optional from value (default None)
        to_value (int): Specify the optional to value (default None)
        limit (int): Specify the optional limit (default 100)
        alert_id (int): Specify the optional alert id (default None)
        dashboard_id (int): Specify the optional dashboard id (default None)
        panel_id (int): Specify the optional panel_id (default None)
        user_id (int): Specify the optional user id (default None)
        type (str): Specify the optional type e.g. alert or annotation (default None)
        tags (list): Specify the optional tags (default None)
    """

    from_value: int = None
    to_value: int = None
    limit: int = 100
    alert_id: int = None
    dashboard_id: int = None
    panel_id: int = None
    user_id: int = None
    type: str = None
    tags: list = None


@dataclass
class AnnotationObject:
    """The class includes all necessary variables to generate an annotation object

    Args:
        time (int): Specify the time as number in milliseconds
        time_end (int): Specify the end time as number in milliseconds
        tags (list): Specify the organization annotation tags from a data source that are not connected specifically to a dashboard or panel
        text (str): Specify the annotation description message
        dashboard_uid (str): Specify the optional dashboard_uid (default None)
        panel_id (int): Specify the optional panel_id (default None)
    """

    time: int
    time_end: int
    tags: list
    text: str
    dashboard_uid: str = None
    panel_id: int = None


@dataclass
class AnnotationGraphiteObject:
    """The class includes all necessary variables to generate a Graphite annotation object

    Args:
        what (str): Specify the event of the annotation
        tags (list): Specify the organization annotation tags from a data source that are not connected specifically to a dashboard or panel
        when (int): Specify the optional time as number in milliseconds
        data (str): Specify the optional annotation description message
    """

    what: str
    tags: list
    when: int = None
    data: str = None


@dataclass
class GlobalUser:
    """The class includes all necessary variables to generate a global user object

    Args:
        name (str): Specify the name of the user
        email (str): Specify the email of the user
        login (str): Specify the login type of the user
        password (str): Specify the password of the user
        org_id (int): Specify the optional org id of the user (default None)
    """

    name: str
    email: str
    login: str
    password: str
    org_id: int = None


@dataclass
class RolePermission:
    """The class includes all necessary variables to generate a role permission object

    Args:
        action (str): Specify the custom role action definition
        scope (str): Specify the scope definition. If not present, no scope will be mapped to the permission (default None)
    """

    action: str
    scope: str = None


@dataclass
class CustomRole:
    """The class includes all necessary variables to generate a custom role object

    Args:
        name (str): Specify the name of the role
        uid (str): Specify the optional uid of the role (default None)
        global_role (bool): Specify the if the role is global or not. If set to False, the default org id of the authenticated user will be used from the request (default False)
        version (int): Specify the optional version of the role (default None)
        description (str): Specify the optional description of the role (default None)
        display_name (str): Specify the optional display_name of the role (default None)
        group (str): Specify the optional org group of the role (default None)
        hidden (bool): Specify whether the role is hidden or not.  If set to True, then the role does not show in the role picker. It will not be listed by API endpoints unless explicitly specified (default False)
        permissions (list): Specify the optional permissions of the role as a list of the RolePermission objects (default None)
    """

    name: str
    uid: str = None
    global_role: bool = False
    version: int = None
    description: str = None
    display_name: str = None
    group: str = None
    hidden: bool = False
    permissions: List[RolePermission] = None


@dataclass
class DatasourceCache:
    """The class includes all necessary variables to generate a datasource cache object

    Args:
        datasource_id (int): Specify the datasource id
        datasource_uid (str): Specify the datasource uid
        enabled (bool): Specify if caching should be enabled for the datasource
        use_default_ttl (bool): Specify if the configured default TTL (Time-To-Live) should be used for both query and resource caching, instead of the user-specified values
        ttl_queries_ms (int): Specify the TTL to use for query caching, in milliseconds
        ttl_resources_ms (int): Specify the TTL to use for resource caching, in milliseconds
    """

    datasource_id: int
    datasource_uid: str
    enabled: bool
    use_default_ttl: bool
    ttl_queries_ms: int
    ttl_resources_ms: int


@dataclass
class PublicDashboard:
    """The class includes all necessary variables to generate a public dashboard object

    Args:
        uid (str): Specify the optional unique identifier when creating a public dashboard. If it’s none, it will generate a new uid (default None)
        access_token (str): Specify the optional unique access token. If it’s none, it will generate a new access token (default None)
        time_selection_enabled (bool): Specify the optional enablement of the time picker inside the public dashboard (default False)
        is_enabled (bool): Specify the optional enablement of the public dashboard (default False)
        annotations_enabled (bool): Specify the optional enablement of the annotations inside the public dashboard (default False)
        share (str): Specify the optional share mode of the public dashboard (default public)
    """

    uid: str = None
    access_token: str = None
    time_selection_enabled: bool = False
    is_enabled: bool = False
    annotations_enabled: bool = False
    share: str = "public"


@dataclass
class SSOSetting:
    """The class includes all necessary variables to generate an SSO setting object

    Args:
        api_url (str): Specify the SSO api url
        client_id (str): Specify the SSO client id
        client_secret (str): Specify the SSO client secret
        enabled (bool): Specify if the SSO provider is enabled or not
        scopes (str): Specify the SSO scopes
    """

    api_url: str
    client_id: str
    client_secret: str
    enabled: bool
    scopes: str


@dataclass
class DashboardSchema:
    """The class includes all necessary variables to generate a dashboard schema object that is used for the reporting functionality

    Args:
        dashboard_uid (str): Specify the dashboard uid
        time_range_from (str): Specify the dashboard time range from
        time_range_to (str): Specify the dashboard time range to
        report_variables (dict): Specify the key-value pairs containing the template variables for this report, in dict format. If the value is None, the template variables from the reports dashboard will be used (default None)
    """

    dashboard_uid: str
    time_range_from: str
    time_range_to: str
    report_variables: dict = None


@dataclass
class Report:
    """The class includes all necessary variables to generate a report object

    Args:
        name (str): Specify the name of the report that is used as an email subject
        recipients (str): Specify the comma-separated list of emails to which to send the report to
        reply_to (str): Specify the comma-separated list of emails used in a reply-to field of the report email
        message (str): Specify the text message used for the body of the report email
        start_date (str): Specify the distribution starts from this date
        end_date (str): Specify the distribution end from this date
        time_zone (str): Specify the time zone used to schedule report execution
        orientation (str): Specify if the orientation should be portrait or landscape
        layout (str): Specify if the layout should be grid or simple
        enable_dashboard_url (str): Specify if the dashboard url should be added to the bottom of the report email
        dashboards (List[DashboardSchema]): Specify the dashboards for which the reports should be generated
        frequency (str): Specify how often the report should be sent. Can be once, hourly, daily, weekly, monthly, last or custom. The value last schedules the report for the last day of the month. The value custom schedules the report to be sent on a custom interval. It requires interval_frequency and interval_amount to be specified e.g. every 2 weeks, where 2 is an interval_amount and weeks is an interval_frequency (default last)
        interval_frequency (str): Specify the type of the custom interval hours, days, weeks, months (default None)
        interval_amount (int): Specify the interval amount of the custom type (default 0)
        workdays_only (bool): Specify if the report only on Monday-Friday should be sent. Applicable to hourly and daily types of schedule (default None)
        formats (List[str]): Specify what kind of attachment to generate for the report. Available report formats are csv, pdf and image. The type csv attaches a CSV file for each table panel and the type image embeds an image of a dashboard into the emails body (default List["pdf"])
    """

    name: str
    recipients: str
    reply_to: str
    message: str
    start_date: str
    end_date: str
    time_zone: str
    orientation: str
    layout: str
    enable_dashboard_url: bool
    dashboards: List[DashboardSchema]
    frequency: str = "last"
    interval_frequency: str = None
    interval_amount: int = 0
    workdays_only: bool = None
    formats: List[str] = field(default_factory=lambda: ["pdf"])


@dataclass
class ReportBrandingSettings:
    """The class includes all necessary variables to generate a report branding settings object

    Args:
        report_logo_url (str): Specify the url of an image used as a logo on every page of the report
        email_logo_url (str): Specify the url of an image used as a logo in the email
        email_footer_mode (str): Specify the email footer mode. Can be sent-by or none. The value sent-by adds a 'Sent by email footer text' footer link to the email. Requires specifying values in the email_footer_text and email_footer_link fields. The value none suppresses adding a 'Sent by' footer link to the email
        email_footer_text (str): Specify the text of a URL added to the email 'Sent by' footer (default None)
        email_footer_link (str): Specify the url address value added to the email 'Sent by' footer (default None)
    """

    report_logo_url: str
    email_logo_url: str
    email_footer_mode: str
    email_footer_text: str = None
    email_footer_link: str = None



import logging
from typing import Union
import asyncio

from httpx import AsyncClient, Client, BasicAuth, Response

import json

from .model import APIModel, APIEndpoints
from .api import Api


class OtherHTTP:
    """The class includes all necessary methods to access other Grafana API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_frontend_settings(self) -> dict:
        """The method includes a functionality to get the frontend settings

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding frontend settings
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.FRONTEND.value}/settings"
        )

        if api_call == dict():
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def renew_login_session_based_on_remember_cookie(self):
        """The method includes a functionality to renew the login session based on the remember cookie

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """
        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.LOGIN.value}/ping"
        )

        if api_call.get("message") != "Logged in":
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.info(
                "You successfully renewed the login session based on the cookie."
            )

    def get_health_status(self) -> dict:
        """The method includes a functionality to get the health information

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the health information
        """

        http = Api(self.grafana_api_model).create_the_http_api_client()

        http_result = self._basic_get_call_without_token_auth(
            http, f"{self.grafana_api_model.host}/api/health"
        )

        api_call: dict = json.loads(http_result.text)

        if api_call == dict() or api_call.get("commit") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_metrics(
        self, basic_auth_username: str = None, basic_auth_password: str = None
    ) -> str:
        """The method includes a functionality to get the Grafana metrics information

        Args:
            basic_auth_username (str): Specify the optional basic auth username
            basic_auth_password (str): Specify the optional basic auth password

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (str): Returns the metrics information
        """

        http = Api(self.grafana_api_model).create_the_http_api_client()

        basic_auth = None
        if basic_auth_username is not None and basic_auth_password is not None:
            basic_auth = BasicAuth(basic_auth_username, basic_auth_password)

        api_call: str = self._basic_get_call_without_token_auth(
            http, f"{self.grafana_api_model.host}/metrics", basic_auth
        ).text

        if len(api_call) == 0 or api_call is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_plugin_metrics(
        self,
        plugin_id: str,
        basic_auth_username: str = None,
        basic_auth_password: str = None,
    ) -> str:
        """The method includes a functionality to get the Grafana plugin metrics information

        Args:
            plugin_id (str): Specify the plugin id
            basic_auth_username (str): Specify the optional basic auth username
            basic_auth_password (str): Specify the optional basic auth password

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (str): Returns the metrics information
        """

        http = Api(self.grafana_api_model).create_the_http_api_client()

        basic_auth = None
        if basic_auth_username is not None and basic_auth_password is not None:
            basic_auth = BasicAuth(basic_auth_username, basic_auth_password)

        if len(plugin_id) != 0:
            url: str = f"{self.grafana_api_model.host}/metrics/plugins/{plugin_id}"
            api_call: str = self._basic_get_call_without_token_auth(
                http, url, basic_auth
            ).text

            if len(api_call) == 0 or api_call is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no plugin_id defined.")
            raise ValueError

    def _basic_get_call_without_token_auth(
        self, http: Union[Client, AsyncClient], url: str, basic_auth: BasicAuth = None
    ) -> Response:
        """The method includes a functionality to perform a basic GET call to an endpoint with optional BasicAuth

        Args:
            http (Union[Client, AsyncClient]): Specify the used client
            url (str): Specify the url of the performed api call
            basic_auth (BasicAuth): Specify the optional basic auth credentials (Default None)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (Response): Returns the corresponding result of the api call
        """

        try:
            if self.grafana_api_model.http2_support:

                async def _execute_async_http_requests():
                    async with http:
                        return await http.request("GET", url, auth=basic_auth)

                return asyncio.run(_execute_async_http_requests())
            else:
                return http.request("GET", url, auth=basic_auth)
        except Exception as e:
            raise e



import logging
import json

import httpx

from .api import Api
from .model import APIModel, APIEndpoints, RequestsMethods


class Folder:
    """The class includes all necessary methods to access the Grafana folder API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_folders(self) -> list:
        """The method includes a functionality to extract all folders inside the organization

        Required Permissions:
            Action: folders:read
            Scope: folders:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all folders
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.FOLDERS.value
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Please, check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_folder_by_uid(self, uid: str) -> dict:
        """The method includes a functionality to extract all folder information specified by the uid of the folder

        Args:
            uid (str): Specify the uid of the folder

        Required Permissions:
            Action: folders:read
            Scope: folders:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a folder
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.FOLDERS.value}/{uid}"
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no dashboard uid defined.")
            raise ValueError

    def get_folder_by_id(self, id: int) -> dict:
        """The method includes a functionality to extract all folder information specified by the id of the folder

        Args:
            id (int): Specify the id of the folder

        Required Permissions:
            Action: folders:read
            Scope: folders:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a folder
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.FOLDERS.value}/id/{id}",
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no folder id defined.")
            raise ValueError

    def create_folder(
        self, title: str, uid: str = None, parent_uid: str = None
    ) -> dict:
        """The method includes a functionality to create a new folder inside the organization specified by the defined title and the optional uid

        Args:
            title (str): Specify the title of the folder
            uid (str): Specify the uid of the folder (default None)
            parent_uid (str): Specify the parent_uid of the folder (default None)

        Required Permissions:
            Action: folders:create, folders:write
            Scope: folders:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns a newly created folder
        """

        if len(title) != 0:
            folder_information: dict = dict()
            folder_information.update({"title": title})

            if uid is not None and len(uid) != 0:
                folder_information.update({"uid": uid})

            if parent_uid is not None and len(parent_uid) != 0:
                folder_information.update({"parentUid": parent_uid})

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.FOLDERS.value,
                RequestsMethods.POST,
                json.dumps(folder_information),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no folder uid or title defined.")
            raise ValueError

    def update_folder(
        self, title: str, uid: str, version: int = 0, overwrite: bool = False
    ) -> dict:
        """The method includes a functionality to update a folder information inside the organization specified by the uid, the title, the version of the folder or if folder information be overwritten

        Args:
            title (str): Specify the title of the folder
            uid (str): Specify the uid of the folder
            version (int): Specify the version of the folder (default 0)
            overwrite (bool): Should the already existing folder information be overwritten (default False)

        Required Permissions:
            Action: folders:write
            Scope: folders:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns an updated folder
        """

        if overwrite is True:
            version = None

        if len(title) != 0 and version != 0 and len(uid) != 0:
            folder_information: dict = dict()
            folder_information.update({"title": title})
            folder_information.update({"overwrite": overwrite})
            folder_information.update({"uid": uid})

            if version is not None:
                folder_information.update({"version": version})

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.FOLDERS.value}/{uid}",
                RequestsMethods.PUT,
                json.dumps(folder_information),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no folder title, version or uid defined.")
            raise ValueError

    def move_folder(self, uid: str, parent_uid: str = None):
        """The method includes a functionality to move a folder inside the organization specified by the defined uid. This feature is only relevant if nested folders are enabled

        Args:
            uid (str): Specify the uid of the folder
            parent_uid (str): Specify the parent_uid of the folder. If the value is None, then the folder is moved under the root (default None)

        Required Permissions:
            Action: folders:create, folders:write
            Scope: folders:*, folders:uid:<destination folder UID>

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the moved folder
        """

        if len(uid) != 0:
            folder_information: dict = dict()

            if parent_uid is not None and len(parent_uid) != 0:
                folder_information.update({"parentUid": parent_uid})

            api_call = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.FOLDERS.value}/{uid}/move",
                RequestsMethods.POST,
                json.dumps(folder_information),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no folder uid defined.")
            raise ValueError

    def delete_folder(self, uid: str):
        """The method includes a functionality to delete a folder inside the organization specified by the defined uid

        Args:
            uid (str): Specify the uid of the folder

        Required Permissions:
            Action: folders:delete
            Scope: folders:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0:
            api_call = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.FOLDERS.value}/{uid}",
                RequestsMethods.DELETE,
            )

            if (
                isinstance(api_call, dict)
                and api_call.get("message") != "Folder deleted"
            ):
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            elif isinstance(api_call, httpx.Response) and api_call.status_code != 200:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully destroyed the folder.")
        else:
            logging.error("There is no folder uid defined.")
            raise ValueError

    def get_folder_permissions(self, uid: str) -> list:
        """The method includes a functionality to extract the folder permissions inside the organization specified by the defined uid

        Args:
            uid (str): Specify the uid of the folder

        Required Permissions:
            Action: folders.permissions:read
            Scope: folders:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns a list of folder permissions
        """

        if len(uid) != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.FOLDERS.value}/{uid}/permissions",
                RequestsMethods.GET,
            )

            if api_call == list() or api_call[0].get("folderId") is None:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no folder uid defined.")
            raise ValueError

    def update_folder_permissions(self, uid: str, permission_json: dict):
        """The method includes a functionality to update the folder permissions based on the specified uid and the permission json document

        Args:
            uid (str): Specify the uid of the folder
            permission_json (dict): Specify the inserted permissions as dict

        Required Permissions:
            Action: folders.permissions:write
            Scope: folders:*

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0 and len(permission_json) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.FOLDERS.value}/{uid}/permissions",
                RequestsMethods.POST,
                json.dumps(permission_json),
            )

            if api_call.get("message") not in [
                "Dashboard permissions updated",
                "Folder permissions updated",
            ]:
                logging.error(f"Please, check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully modified the folder permissions.")
        else:
            logging.error("There is no folder uid or permission json defined.")
            raise ValueError

    def get_folder_id_by_dashboard_path(self, dashboard_path: str) -> int:
        """The method includes a functionality to extract the folder id specified inside model dashboard path

        Args:
            dashboard_path (str): Specify the dashboard path

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            folder_id (int): Returns the folder id
        """

        if dashboard_path.lower() == "general":
            return 0

        if len(dashboard_path) != 0:
            folders: list = self.get_all_folder_ids_and_names()
            folder_id: int = 0

            for f in folders:
                if dashboard_path == f.get("title"):
                    folder_id = f.get("id")

            if folder_id == 0:
                logging.error(
                    f"There's no folder_id for the dashboard named {dashboard_path} available."
                )
                raise Exception

            return folder_id
        else:
            logging.error("There is no dashboard_path defined.")
            raise ValueError

    def get_all_folder_ids_and_names(self) -> list:
        """The method extract all folder id and names inside the complete organisation

        Returns:
            folders (list): Returns a list of dicts with folder ids and the corresponding names
        """

        folders_raw: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.SEARCH.value}?folderIds=0"
        )
        folders_raw_len: int = len(folders_raw)
        folders: list = list()

        for i in range(0, folders_raw_len):
            folders.append(
                {"title": folders_raw[i].get("title"), "id": folders_raw[i].get("id")}
            )

        return folders



import json
import logging

from .api import Api
from .model import APIModel, APIEndpoints, RequestsMethods


class ServiceAccount:
    """The class includes all necessary methods to access the Grafana service account API endpoints. Be aware that the functionality inside the class only works with basic authentication (username and password) and that the authenticated user is a Grafana Admin

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def search_service_account(
        self, results_per_page: int = 1000, pages: int = 1, query: str = None
    ) -> dict:
        """The method includes a functionality to get the service accounts specified by the optional pagination functionality

        Required Permissions:
            Action: serviceaccounts:read
            Scope: global:serviceaccounts:*

        Args:
            results_per_page (int): Specify the results_per_page as integer (default 1000)
            pages (int): Specify the pages as integer (default 1)
            query (str): Specify the query (default None)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the service accounts
        """

        api_request_url: str = (
            f"{APIEndpoints.SERVICE_ACCOUNTS.value}/search?perpage={results_per_page}&page={pages}"
        )

        if query is not None and len(query) != 0:
            api_request_url: str = f"{api_request_url}&query={query}"

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            api_request_url,
        )

        if api_call == dict() or api_call.get("totalCount") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_service_account_by_id(self, id: int) -> dict:
        """The method includes a functionality to get a service account specified by the id

        Required Permissions:
            Action: serviceaccounts:read
            Scope: serviceaccounts:*

        Args:
            id (int): Specify the id of the service account

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the service account
        """

        if id is not None and id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/{id}",
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def create_service_account(self, name: str, role: str) -> dict:
        """The method includes a functionality to create a service account

        Required Permissions:
            Action: serviceaccounts:write
            Scope: serviceaccounts:*

        Args:
            name (str): Specify the name of the service account
            role (str): Specify the role of the service account

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the created service account
        """

        if len(name) != 0 and len(role) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.SERVICE_ACCOUNTS.value,
                RequestsMethods.POST,
                json.dumps(dict({"name": name, "role": role})),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no name or role defined.")
            raise ValueError

    def update_service_account(self, id: int, name: str, role: str) -> dict:
        """The method includes a functionality to update a service account specified by the id, name and role

        Required Permissions:
            Action: serviceaccounts:write
            Scope: serviceaccounts:*

        Args:
            id (int): Specify the id of the service account
            name (str): Specify the name of the service account
            role (str): Specify the role of the service account

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the service account
        """

        if id is not None and id != 0 and len(name) != 0 and len(role) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/{id}",
                RequestsMethods.PATCH,
                json.dumps(dict({"name": name, "role": role})),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id, name or role defined.")
            raise ValueError

    def delete_service_account(self, id: int):
        """The method includes a functionality to delete a service account specified by the id

        Required Permissions:
            Action: serviceaccounts:delete
            Scope: serviceaccounts:id:*

        Args:
            id (int): Specify the id of the service account

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id is not None and id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/{id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Service account deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the service account.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def get_service_account_tokens_by_id(self, id: int) -> list:
        """The method includes a functionality to get a service account tokens specified by the id

        Required Permissions:
            Action: serviceaccounts:read
            Scope: serviceaccounts:*

        Args:
            id (int): Specify the id of the service account

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the service account tokens
        """

        if id is not None and id != 0:
            api_call: list = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/{id}/tokens",
            )

            if api_call != list() and api_call[0].get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def create_service_account_token_by_id(self, id: int, name: str, role: str) -> dict:
        """The method includes a functionality to create a service account token specified by the id

        Required Permissions:
            Action: serviceaccounts:write
            Scope: serviceaccounts:*

        Args:
            id (int): Specify the id of the service account
            name (str): Specify the name of the service account
            role (str): Specify the role of the service account

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the service account token
        """

        if id is not None and id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/{id}/tokens",
                RequestsMethods.POST,
                json.dumps(dict({"name": name, "role": role})),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id, name or role defined.")
            raise ValueError

    def delete_service_account_token_by_id(self, id: int, token_id: int):
        """The method includes a functionality to delete a service account token specified by the id

        Required Permissions:
            Action: serviceaccounts:write
            Scope: serviceaccounts:*

        Args:
            id (int): Specify the id of the service account
            token_id (int):

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id is not None and id != 0 and token_id is not None and token_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/{id}/tokens/{token_id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Service account token deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the service account token.")
        else:
            logging.error("There is no id or token_id defined.")
            raise ValueError

    def migrate_api_keys_to_service_accounts(self):
        """The method includes a functionality to migrate all api keys to service accounts

        Required Permissions:
            Action: serviceaccounts:write
            Scope: serviceaccounts:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.SERVICE_ACCOUNTS.value}/migrate",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if (
            api_call.get("migrated") is None
            and api_call.get("message") != "API keys migrated to service accounts"
        ):
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.info(
                "You successfully migrated the API keys to the service accounts."
            )

    def migrate_api_key_to_service_account(self, key_id: int):
        """The method includes a functionality to migrate an api key to a service account specified by the key id

        Required Permissions:
            Action: serviceaccounts:write
            Scope: serviceaccounts:*

        Args:
            key_id (int): Specify the api key id of the api key

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if key_id is not None and key_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/migrate/{key_id}",
                RequestsMethods.POST,
                json.dumps(dict()),
            )

            if api_call.get("message") != "Service accounts migrated":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info(
                    "You successfully migrated the api key to an service account."
                )
        else:
            logging.error("There is no key_id defined.")
            raise ValueError

    def get_service_account_migration_status(self) -> bool:
        """The method includes a functionality to get the corresponding api key migration status

        Required Permissions:
            Action: serviceaccounts:read
            Scope: serviceaccounts:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns the api key migration status
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.SERVICE_ACCOUNTS.value}/migrationstatus",
        )

        if api_call == dict() or api_call.get("migrated") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call.get("migrated")

    def hide_the_api_keys_tab(self):
        """The method includes a functionality to hide the api keys tab inside the UI

        Required Permissions:
            Action: serviceaccounts:write
            Scope: serviceaccounts:*

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.SERVICE_ACCOUNTS.value}/hideApiKeys",
            RequestsMethods.POST,
            json.dumps(dict()),
        )

        if api_call.get("message") != "API keys hidden":
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            logging.info("You successfully hide the api keys tab inside the UI.")

    def revert_service_account_token_to_api_key(self, id: int, key_id: int):
        """The method includes a functionality to revert a service account and transform it to the legacy api token specified by the service account id and the key id

        Required Permissions:
            Action: serviceaccounts:delete
            Scope: serviceaccounts:id:*

        Args:
            id (int): Specify the id of the service account
            key_id (int): Specify the api key id of the api key

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id is not None and id != 0 and key_id is not None and key_id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.SERVICE_ACCOUNTS.value}/{id}/revert/{key_id}",
                RequestsMethods.POST,
                json.dumps(dict()),
            )

            if api_call.get("message") != "reverted service account to API key":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully reverted the service account.")
        else:
            logging.error("There is no id or key_id defined.")
            raise ValueError



import json
import logging

from .model import APIModel, APIEndpoints, SortDirection, RequestsMethods
from .api import Api


class Library:
    """The class includes all necessary methods to access the Grafana library API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_all_library_elements(
        self,
        results_per_page: int = 100,
        pages: int = 1,
        search_string: str = None,
        kind: int = 1,
        sort_direction: SortDirection = SortDirection.DESC,
        types_filter: str = None,
        exclude_uid: str = None,
        folder_filter_ids: str = None,
    ) -> dict:
        """The method includes a functionality to get a list of all library elements the authenticated user has permission to view. Use the perPage query parameter to control the maximum number of library elements returned, The default limit is 100. You can also use the page query parameter to fetch library elements from any page other than the first one

         Args:
            results_per_page (int): Specify the results_per_page as integer (default 100)
            pages (int): Specify the pages as integer (default 1)
            search_string (str): Specify the search string (default None)
            kind (int): Specify the kind of element to search for. Use 1 for library panels or 2 for library variables (default 1)
            sort_direction (SortDirection): Specify the sort order of elements. Use alpha-asc for ascending and alpha-desc for descending sort order (default alpha-desc)
            types_filter (str): Specify a comma separated list of types to filter the elements by (default None)
            exclude_uid (str): Specify the element uid to exclude from search results (default None)
            folder_filter_ids (str): Specify a comma separated list of folder ID(s) to filter the elements by (default None)

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the library elements
        """

        additional_parameters: str = ""
        if search_string is not None and len(search_string) != 0:
            additional_parameters += f"&searchString={search_string}"

        if types_filter is not None and len(types_filter) != 0:
            additional_parameters += f"&typeFilter={types_filter}"

        if exclude_uid is not None and len(exclude_uid) != 0:
            additional_parameters += f"&excludeUid={exclude_uid}"

        if folder_filter_ids is not None and len(folder_filter_ids) != 0:
            additional_parameters += f"&folderFilter={folder_filter_ids}"

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.LIBRARY.value}?perpage={results_per_page}&page={pages}&kind={kind}"
            f"&sortDirection={sort_direction}{additional_parameters}",
        )

        if api_call == dict() or api_call.get("result") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_library_element_by_uid(self, uid: str) -> dict:
        """The method includes a functionality to get a library element with the given uid

         Args:
            uid (str): Specify the uid of the library element

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding library element
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LIBRARY.value}/{uid}",
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def get_library_element_by_name(self, name: str) -> dict:
        """The method includes a functionality to get a library element with the given name

         Args:
            name (str): Specify the name of the library element

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding library element
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LIBRARY.value}/name/{name}",
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def get_library_element_connections(self, uid: str) -> dict:
        """The method includes a functionality to get a list of connections for a library element based on the specified uid

         Args:
            uid (str): Specify the uid of the library element

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the corresponding list of connections
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LIBRARY.value}/{uid}/connections",
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def create_library_element(
        self,
        folder_id: int,
        model: dict,
        kind: int = 1,
        folder_uid: str = None,
        name: str = None,
        uid: str = None,
    ) -> dict:
        """The method includes a functionality to create a library element based on the specified folder id and model

         Args:
             folder_id (int): Specify the folder where the library element is stored. It is deprecated since Grafana v9
             model (dict): Specify the JSON model for the library element
             kind (int): Specify the kind of element to search for. Use 1 for library panels or 2 for library variables (default 1)
             folder_uid (str): Specify the uid of the folder where the library element is stored. Specify an empty string when it is general folder (default None)
             name (str): Specify the name of the library element (default None)
             uid (str): Specify the uid of the library element (default None)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the newly created library element
        """

        if (
            folder_id != 0
            and model is not None
            and model != dict()
            and kind is not None
            and kind != 0
        ):
            request_parameters: dict = dict(
                {"folderId": folder_id, "model": model, "kind": kind}
            )

            if folder_uid is not None and len(folder_uid) != 0:
                request_parameters.update({"folderUid": folder_uid})

            if name is not None and len(name) != 0:
                request_parameters.update({"name": name})

            if uid is not None and len(uid) != 0:
                request_parameters.update({"uid": uid})

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.LIBRARY.value,
                RequestsMethods.POST,
                json.dumps(request_parameters),
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no folder_id, kind or model defined.")
            raise ValueError

    def update_library_element(
        self,
        uid: str,
        folder_id: int,
        folder_uid: str,
        name: str,
        model: dict,
        version: int,
        kind: int = 1,
    ) -> dict:
        """The method includes a functionality to update a library element

         Args:
             uid (str): Specify the uid of the library element
             folder_id (int): Specify the folder where the library element is stored. It is deprecated since Grafana v9
             folder_uid (str): Specify the uid of the folder where the library element is stored. Specify an empty string when it is general folder
             name (str): Specify the name of the library element
             model (dict): Specify the JSON model for the library element
             version (int): Specify the version for the library element
             kind (int): Specify the kind of element to search for. Use 1 for library panels or 2 for library variables (default 1)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the updated library element
        """

        if (
            len(uid) != 0
            and folder_id != 0
            and len(folder_uid) != 0
            and len(name) != 0
            and model is not None
            and model != dict()
            and version != 0
            and kind is not None
            and kind != 0
        ):
            request_parameters: dict = dict(
                {
                    "uid": uid,
                    "folderId": folder_id,
                    "folderUid": folder_uid,
                    "name": name,
                    "model": model,
                    "version": version,
                    "kind": kind,
                }
            )

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LIBRARY.value}/{uid}",
                RequestsMethods.PATCH,
                json.dumps(request_parameters),
            )

            if api_call == dict() or api_call.get("result") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error(
                "There is no folder_id, folder_uid, name, version, kind or model defined."
            )
            raise ValueError

    def delete_library_element(self, uid: str):
        """The method includes a functionality to delete a library element specified by the uid

         Args:
             uid (str): Specify the uid of the library element

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.LIBRARY.value}/{uid}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Library element deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info(
                    "You successfully deleted the corresponding Library element."
                )
        else:
            logging.error("There is no uid defined.")
            raise ValueError



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods
from .api import Api


class ShortUrl:
    """The class includes all necessary methods to access the Grafana short url API endpoint

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def create_short_url(self, path: str):
        """The method includes a functionality to create a short link for a specific dashboard

        Args:
            path (str): Specify the corresponding dashboard path

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the uid and the url of the newly generated link
        """

        if len(path) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.SHORT_URLS.value,
                RequestsMethods.POST,
                json.dumps(dict({"path": path})),
            )

            if api_call == dict() or api_call.get("url") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no path defined.")
            raise ValueError



import json
import logging
from typing import Union

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    Report,
    ReportBrandingSettings,
)
from .api import Api


class Reporting:
    """The class includes all necessary methods to access the Grafana reporting API endpoints. Be aware that the functionality is a Grafana ENTERPRISE v7.0+ feature

    HINT: Note Grafana Enterprise API need required permissions if fine-grained access control is enabled

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_reports(self) -> list:
        """The method includes a functionality to get all reports

        Required Permissions:
            Action: reports:read
            Scope: [reports:*, reports:id:*]

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all reports
        """

        api_call: Union[list, dict] = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.REPORTING.value,
            response_status_code=True,
        )

        status_code: int = (
            api_call[0].get("status")
            if isinstance(api_call, list)
            else api_call.get("status")
        )

        reporting_status_dict: dict = dict(
            {
                401: "Authentication failed.",
                500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
            }
        )

        if status_code == 200:
            return api_call
        elif 401 <= status_code <= 500:
            logging.error(reporting_status_dict.get(status_code))
            raise Exception
        else:
            logging.error(f"Check the error: {api_call}.")
            raise Exception

    def get_report_by_id(self, id: int) -> dict:
        """The method includes a functionality to get a report specified by the report id

        Args:
            id (int): Specify the report id

        Required Permissions:
            Action: reports:read
            Scope: [reports:*, reports:id:*, reports:id:<report_id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the report
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.REPORTING.value}/{id}",
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            reporting_status_dict: dict = dict(
                {
                    400: "Bad request (invalid report ID).",
                    401: "Authentication failed.",
                    403: "Forbidden (access denied to a report or a dashboard used in the report).",
                    404: "Not found (such report does not exist).",
                    500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
                }
            )

            if status_code == 200:
                return api_call
            elif 400 <= status_code <= 500:
                logging.error(reporting_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def create_report(self, report: Report) -> int:
        """The method includes a functionality to create a report

        Args:
            report (Report): Specify the report object

        Required Permissions:
            Action: reports:create
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (int): Returns the report id
        """

        report_object: Union[bool, dict] = self._validate_report_object(report)

        if isinstance(report_object, dict):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.REPORTING.value,
                RequestsMethods.POST,
                json.dumps(report_object),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            reporting_status_dict: dict = dict(
                {
                    400: "Bad request.",
                    403: "Forbidden (access denied to a report or a dashboard used in the report).",
                    500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Report created":
                return int(api_call.get("id"))
            elif 400 <= status_code <= 500:
                logging.error(reporting_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no report object defined.")
            raise ValueError

    def update_report(self, id: int, report: Report):
        """The method includes a functionality to update a report

        Args:
            id (int): Specify the report id
            report (Report): Specify the report object

        Required Permissions:
            Action: reports:write
            Scope: [reports:*, reports:id:*, reports:id:<report_id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        report_object: Union[bool, dict] = self._validate_report_object(report)

        if id != 0 and isinstance(report_object, dict):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.REPORTING.value}/{id}",
                RequestsMethods.PUT,
                json.dumps(report_object),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            reporting_status_dict: dict = dict(
                {
                    400: "Bad request.",
                    401: "Authentication failed.",
                    403: "Forbidden (access denied to a report or a dashboard used in the report).",
                    404: "Not found (such report does not exist).",
                    500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Report updated":
                logging.info("You successfully updated the report.")
            elif 400 <= status_code <= 500:
                logging.error(reporting_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no id or report object defined.")
            raise ValueError

    def delete_report(self, id: int):
        """The method includes a functionality to delete a report specified by the report id

        Args:
            id (int): Specify the report id

        Required Permissions:
            Action: reports:delete
            Scope: [reports:*, reports:id:*, reports:id:<report_id>]

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.REPORTING.value}/{id}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            reporting_status_dict: dict = dict(
                {
                    400: "Bad request (invalid report ID).",
                    401: "Authentication failed.",
                    404: "Not found (such report does not exist).",
                    500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
                }
            )

            if (
                status_code == 200
                and api_call.get("message") == "Report config was removed"
            ):
                logging.info("You successfully deleted the report.")
            elif 400 <= status_code <= 500:
                logging.error(reporting_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def send_report(
        self, id: int, emails: str = None, use_emails_from_report: bool = None
    ):
        """The method includes a functionality to send a report to a specified email addresses

        Args:
            id (int): Specify the id of forwarded report. It is the same as in the URL when editing a report, not to be confused with the id of the dashboard.
            emails (str): Specify the comma-separated list of emails to which to send the report to. Overrides the emails from the report. Required if useEmailsFromReport is not present (default None)
            use_emails_from_report (bool): Specify if the emails inside the report should be used. Required if emails is not present (default None)

        Required Permissions:
            Action: reports:send
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0 and (
            emails is not None
            and (len(emails) != 0)
            or (use_emails_from_report is not None and use_emails_from_report)
        ):
            if emails is not None and len(emails) != 0:
                result: dict = dict({"id": id, "emails": emails})
            else:
                result: dict = dict(
                    {"id": id, "useEmailsFromReport": use_emails_from_report}
                )

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.REPORTING.value}/email",
                RequestsMethods.POST,
                json.dumps(result),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            reporting_status_dict: dict = dict(
                {
                    400: "Bad request (invalid report ID).",
                    401: "Authentication failed.",
                    403: "Forbidden (access denied to a report or a dashboard used in the report).",
                    404: "Report not found.",
                    500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Report was sent":
                logging.info("You successfully send the report.")
            elif 400 <= status_code <= 500:
                logging.error(reporting_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no id, emails or useEmailsFromReport defined.")
            raise ValueError

    def get_report_branding_settings(self) -> dict:
        """The method includes a functionality to get the report branding settings

        Required Permissions:
            Action: reports.settings:read
            Scope: N/A

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the report branding settings
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.REPORTING.value}/settings",
            response_status_code=True,
        )

        status_code: int = api_call.get("status")

        reporting_status_dict: dict = dict(
            {
                401: "Authentication failed.",
                500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
            }
        )

        if api_call != dict() and api_call.get("id") is not None:
            return api_call
        elif 401 <= status_code <= 500:
            logging.error(reporting_status_dict.get(status_code))
            raise Exception
        else:
            logging.error(f"Check the error: {api_call}.")
            raise Exception

    def save_report_branding_settings(self, branding_settings: ReportBrandingSettings):
        """The method includes a functionality to save the report branding settings

        Args:
            branding_settings (ReportBrandingSettings): Specify the report branding settings object.

        Required Permissions:
            Action: reports.settings:write
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if (
            branding_settings is not None
            and len(branding_settings.report_logo_url) != 0
            and len(branding_settings.email_logo_url) != 0
            and len(branding_settings.email_footer_mode) != 0
        ):
            report_branding_settings_object: dict = dict(
                {
                    "branding": dict(
                        {
                            "reportLogoUrl": branding_settings.report_logo_url,
                            "emailLogoUrl": branding_settings.email_logo_url,
                            "emailFooterMode": branding_settings.email_footer_mode,
                        }
                    ),
                }
            )

            if branding_settings.email_footer_mode == "sent-by":
                if (
                    len(branding_settings.email_footer_text) != 0
                    and len(branding_settings.email_footer_link) != 0
                ):
                    report_branding_settings_object.update(
                        dict(
                            {
                                "branding": dict(
                                    {
                                        "emailFooterText": branding_settings.email_footer_text,
                                        "emailFooterLink": branding_settings.email_footer_link,
                                    }
                                )
                            }
                        )
                    )
                else:
                    logging.error(
                        "There is no correct value for the branding_settings email_footer_text or email_footer_link "
                        "defined."
                    )
                    raise ValueError

            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.REPORTING.value}/settings",
                RequestsMethods.POST,
                json.dumps(report_branding_settings_object),
                response_status_code=True,
            )

            status_code: int = api_call.get("status")

            reporting_status_dict: dict = dict(
                {
                    400: "Bad request (invalid report ID).",
                    401: "Authentication failed.",
                    500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
                }
            )

            if (
                status_code == 200
                and api_call.get("message") == "Report settings saved"
            ):
                logging.info("You successfully saved the report branding settings.")
            elif 400 <= status_code <= 500:
                logging.error(reporting_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no branding_settings defined.")
            raise ValueError

    def send_report_test_email(self, report: Report):
        """The method includes a functionality to send a test report email

        Args:
            report (Report): Specify the report object

        Required Permissions:
            Action: reports: send
            Scope: N/A

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        report_object: Union[bool, dict] = self._validate_report_object(report)

        if isinstance(report_object, dict):
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.REPORTING.value}/test-email",
                RequestsMethods.POST,
                json.dumps(report_object),
            )

            status_code: int = api_call.get("status")

            reporting_status_dict: dict = dict(
                {
                    400: "Bad request.",
                    401: "Authentication failed.",
                    403: "Forbidden (access denied to a report or a dashboard used in the report).",
                    500: "Unexpected error or server misconfiguration. Refer to server logs for more details.",
                }
            )

            if status_code == 200 and api_call.get("message") == "Test email sent":
                logging.info("You successfully sent the test email.")
            elif 400 <= status_code <= 500:
                logging.error(reporting_status_dict.get(status_code))
                raise Exception
            else:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
        else:
            logging.error("There is no report object defined.")
            raise ValueError

    @staticmethod
    def _validate_report_object(report: Report) -> Union[bool, dict]:
        if (
            report is not None
            and len(report.name) != 0
            and len(report.recipients) != 0
            and len(report.reply_to) != 0
            and len(report.message) != 0
            and len(report.start_date) != 0
            and len(report.end_date) != 0
            and len(report.time_zone) != 0
            and len(report.orientation) != 0
            and len(report.layout) != 0
            and report.enable_dashboard_url is not None
            and report.dashboards is not None
            and len(report.frequency) != 0
            and report.formats is not None
        ):
            report_object: dict = dict(
                {
                    "name": report.name,
                    "recipients": report.recipients,
                    "replyTo": report.reply_to,
                    "message": report.message,
                    "schedule": dict(
                        {
                            "startDate": report.start_date,
                            "endDate": report.end_date,
                            "timeZone": report.time_zone,
                            "frequency": report.frequency,
                        }
                    ),
                    "options": dict(
                        {
                            "orientation": report.orientation,
                            "layout": report.layout,
                        }
                    ),
                    "enableDashboardUrl": report.enable_dashboard_url,
                    "formats": report.formats,
                }
            )

            if report.frequency == "custom":
                if len(report.interval_frequency) != 0 and report.interval_amount != 0:
                    report_object.update(
                        dict(
                            {
                                "schedule": dict(
                                    {
                                        "intervalFrequency": report.interval_frequency,
                                        "intervalAmount": report.interval_amount,
                                    }
                                )
                            }
                        )
                    )
                else:
                    logging.error(
                        "There is no interval_frequency or interval_amount defined."
                    )
                    raise ValueError
            elif report.frequency == "hourly" or report.frequency == "daily":
                if report.workdays_only is not None:
                    report_object.update(
                        dict({"schedule": dict({"workdaysOnly": report.workdays_only})})
                    )
                else:
                    logging.error("There is no workdays_only defined.")
                    raise ValueError

            dashboard_list: list = list()
            for dashboard in report.dashboards:
                dashboard_object: dict = dict()

                if len(dashboard.dashboard_uid) != 0:
                    dashboard_object.update(
                        {"dashboard": dict({"uid": dashboard.dashboard_uid})}
                    )
                else:
                    logging.error("There is no dashboard_uid value defined.")
                    raise ValueError

                if (
                    len(dashboard.time_range_from) != 0
                    and len(dashboard.time_range_to) != 0
                ):
                    dashboard_object.update(
                        {
                            "timeRange": dict(
                                {
                                    "from": dashboard.time_range_from,
                                    "to": dashboard.time_range_to,
                                }
                            )
                        }
                    )
                else:
                    logging.error(
                        "There is no time_range_from or time_range_to value defined."
                    )
                    raise ValueError

                if dashboard.report_variables is not None:
                    if dashboard.report_variables != dict():
                        dashboard_object.update(
                            {"reportVariables": dashboard.report_variables}
                        )
                    else:
                        logging.error(
                            "There is no correct value of the report_variables defined."
                        )
                        raise ValueError

                dashboard_list.append(dashboard_object)
            report_object.update(dict({"dashboards": dashboard_list}))

            return report_object
        else:
            return False



import json
import logging

from .model import APIModel, APIEndpoints, RequestsMethods
from .api import Api


class AlertingNotifications:
    """The class includes all necessary methods to access the Grafana alerting notifications API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_all_notification_channels(self) -> list:
        """The method includes a functionality to get all alerting notification channels

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all notification channels
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            APIEndpoints.ALERT_NOTIFICATIONS.value,
            RequestsMethods.GET,
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_all_notification_channels_lookup(self) -> list:
        """The method includes a functionality to lookup and get reduced information of all alerting notification channels

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all notification channels as reduced information
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/lookup",
            RequestsMethods.GET,
        )

        if api_call == list() or api_call[0].get("id") is None:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_notification_channel_by_uid(self, uid: str) -> dict:
        """The method includes a functionality to get an alerting notification channel specified by the uid

        Args:
            uid (str): Specify the uid of the notification channel

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the specified notification channel
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/uid/{uid}",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def get_notification_channel_by_id(self, id: int) -> dict:
        """The method includes a functionality to get an alerting notification channel specified by the id

        Args:
            id (int): Specify the id of the notification channel

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the specified notification channel
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/{id}",
                RequestsMethods.GET,
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def create_notification_channel(self, notification_channel: dict) -> dict:
        """The method includes a functionality to create an alerting notification channel specified by the notification channel dict

        Args:
            notification_channel (dict): Specify the channel of the notification

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the newly created notification channel
        """

        if notification_channel != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                APIEndpoints.ALERT_NOTIFICATIONS.value,
                RequestsMethods.POST,
                json.dumps(notification_channel),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no notification_channel defined.")
            raise ValueError

    def update_notification_channel_by_uid(
        self, uid: str, notification_channel: dict
    ) -> dict:
        """The method includes a functionality to update an alerting notification channel specified by the notification channel dict and the uid

        Args:
            uid (str): Specify the uid of the notification channel
            notification_channel (dict): Specify the channel of the notification

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the updated notification channel
        """

        if len(uid) != 0 and notification_channel != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/uid/{uid}",
                RequestsMethods.PUT,
                json.dumps(notification_channel),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid or notification_channel defined.")
            raise ValueError

    def update_notification_channel_by_id(
        self, id: int, notification_channel: dict
    ) -> dict:
        """The method includes a functionality to update an alerting notification channel specified by the notification channel dict and the id

        Args:
            id (int): Specify the id of the notification channel
            notification_channel (dict): Specify the channel of the notification

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the updated notification channel
        """

        if id != 0 and notification_channel != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/{id}",
                RequestsMethods.PUT,
                json.dumps(notification_channel),
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no id or notification_channel defined.")
            raise ValueError

    def delete_notification_channel_by_uid(self, uid: str):
        """The method includes a functionality to delete an alerting notification channel specified by the uid

        Args:
            uid (uid): Specify the uid of the notification channel

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/uid/{uid}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Notification deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully destroyed the notification channel.")
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def delete_notification_channel_by_id(self, id: int):
        """The method includes a functionality to delete an alerting notification channel specified by the id

        Args:
            id (int): Specify the id of the notification channel

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if id != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/{id}",
                RequestsMethods.DELETE,
            )

            if api_call.get("message") != "Notification deleted":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully destroyed the notification channel.")
        else:
            logging.error("There is no id defined.")
            raise ValueError

    def test_notification_channel(self, notification_channel: dict):
        """The method includes a functionality to test an alerting notification channel specified by the notification_channel

        Args:
            notification_channel (dict): Specify the channel of the notification

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if notification_channel != dict():
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERT_NOTIFICATIONS.value}/test",
                RequestsMethods.POST,
                json.dumps(notification_channel),
            )

            if api_call.get("message") != "Test notification sent":
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully tested the notification channel.")
        else:
            logging.error("There is no notification_channel defined.")
            raise ValueError



import json
import logging
from typing import List

from .model import (
    APIModel,
    APIEndpoints,
    RequestsMethods,
    AlertRule,
    AlertQuery,
    AlertRuleQueryModel,
    AlertRuleQueryModelCondition,
    EmbeddedContactPoint,
    Route,
    Matcher,
    MuteTimeInterval,
    TimeInterval,
    TimeRange,
)
from .api import Api


class AlertingProvisioning:
    """The class includes all necessary methods to access the Grafana alerting provisioning API endpoints

    Args:
        grafana_api_model (APIModel): Inject a Grafana API model object that includes all necessary values and information

    Attributes:
        grafana_api_model (APIModel): This is where we store the grafana_api_model
    """

    def __init__(self, grafana_api_model: APIModel):
        self.grafana_api_model = grafana_api_model

    def get_alert_rule(self, uid: str) -> dict:
        """The method includes a functionality to get the alert rule specified by the uid

        Args:
            uid (str): Specify the alert rule uid

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the alert rule
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/alert-rules/{uid}",
            )

            if api_call == dict() or api_call.get("id") is None:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def add_alert_rule(self, alert_rule: AlertRule, disable_provenance: bool = False):
        """The method includes a functionality to create a new alert rule

        Args:
            alert_rule (AlertRule): Specify the alert rule
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if alert_rule is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/alert-rules",
                RequestsMethods.POST,
                json.dumps(self._create_alert_rule_dictionary(alert_rule)),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully created the corresponding alert rule.")
        else:
            logging.error("There is no alert_rule defined.")
            raise ValueError

    def update_alert_rule(
        self, uid: str, alert_rule: AlertRule, disable_provenance: bool = False
    ):
        """The method includes a functionality to update an existing alert rule

        Args:
            uid (str): Specify the alert rule uid
            alert_rule (AlertRule): Specify the alert rule
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0 and alert_rule is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/alert-rules/{uid}",
                RequestsMethods.PUT,
                json.dumps(self._create_alert_rule_dictionary(alert_rule)),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the corresponding alert rule.")
        else:
            logging.error("There is no uid or alert_rule defined.")
            raise ValueError

    def update_the_interval_of_a_alert_rule_group(
        self,
        folder_uid: str,
        group: str,
        alert_rule_group_interval: int,
        disable_provenance: bool = False,
    ):
        """The method includes a functionality to update the interval of a alert rule group

        Args:
            folder_uid (str): Specify the folder uid
            group (str): Specify the group
            alert_rule_group_interval (int): Specify the alert rule group interval
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(folder_uid) != 0 and len(group) != 0 and alert_rule_group_interval != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/folder/{folder_uid}/rule-groups/{group}",
                RequestsMethods.PUT,
                json.dumps({"interval": alert_rule_group_interval}),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info(
                    "You successfully updated the corresponding alert rule group interval."
                )
        else:
            logging.error(
                "There is no folder_uid, group or alert_rule_group_interval defined."
            )
            raise ValueError

    def delete_alert_rule(self, uid: str, disable_provenance: bool = False):
        """The method includes a functionality to delete an alert rule

        Args:
            uid (str): Specify the alert rule uid
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call
        Returns:
            None
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/alert-rules/{uid}",
                RequestsMethods.DELETE,
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the corresponding alert rule.")
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def get_all_contact_points(self) -> list:
        """The method includes a functionality to get all contact points

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all contact points
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ALERTING_PROVISIONING.value}/contact-points",
        )

        if api_call == list():
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def add_contact_point(
        self,
        embedded_contact_point: EmbeddedContactPoint,
        disable_provenance: bool = False,
    ):
        """The method includes a functionality to create a contact point

        Args:
            embedded_contact_point (EmbeddedContactPoint): Specify the embedded contact point
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if embedded_contact_point is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/contact-points",
                RequestsMethods.POST,
                json.dumps(
                    {
                        "name": embedded_contact_point.name,
                        "type": embedded_contact_point.type,
                        "settings": embedded_contact_point.settings,
                        "disableResolveMessage": embedded_contact_point.disable_resolve_message,
                        "provenance": embedded_contact_point.provenance,
                        "UID": embedded_contact_point.uid,
                    }
                ),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully created a new contact point.")
        else:
            logging.error("There is no embedded_contact_point defined.")
            raise ValueError

    def update_contact_point(
        self,
        uid: str,
        embedded_contact_point: EmbeddedContactPoint,
        disable_provenance: bool = False,
    ):
        """The method includes a functionality to update a contact point

        Args:
            uid (str): Specify the uid of the contact point
            embedded_contact_point (EmbeddedContactPoint): Specify the embedded contact point
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0 and embedded_contact_point is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/contact-points/{uid}",
                RequestsMethods.PUT,
                json.dumps(
                    {
                        "name": embedded_contact_point.name,
                        "type": embedded_contact_point.type,
                        "settings": embedded_contact_point.settings,
                        "disableResolveMessage": embedded_contact_point.disable_resolve_message,
                        "provenance": embedded_contact_point.provenance,
                        "UID": embedded_contact_point.uid,
                    }
                ),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the contact point.")
        else:
            logging.error("There is no embedded_contact_point or uid defined.")
            raise ValueError

    def delete_contact_point(self, uid: str):
        """The method includes a functionality to delete a contact point

        Args:
            uid (str): Specify the uid of the contact point

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(uid) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/contact-points/{uid}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the contact point.")
        else:
            logging.error("There is no uid defined.")
            raise ValueError

    def get_notification_policies(self) -> dict:
        """The method includes a functionality to get the notification policy tree

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the notification policy tree
        """

        api_call: dict = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ALERTING_PROVISIONING.value}/policies",
            response_status_code=True,
        )

        if 200 <= api_call.get("status") >= 300:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def add_notification_policies(self, route: Route, disable_provenance: bool = False):
        """The method includes a functionality to set the notification policy tree

        Args:
            route (Route): Specify the alert rule routes
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if route is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/policies",
                RequestsMethods.PUT,
                json.dumps(self._create_alert_route_dictionary(route)),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully added the notification policies.")
        else:
            logging.error("There is no route defined.")
            raise ValueError

    def get_all_mute_timings(self) -> list:
        """The method includes a functionality to get all mute timings

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all mute timings
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ALERTING_PROVISIONING.value}/mute-timings",
        )

        if isinstance(api_call, list) is False:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_mute_timing(self, name: str) -> dict:
        """The method includes a functionality to get a mute timings specified by the name

        Args:
            name (str): Specify the mute timing name

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the mute timing
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/mute-timings/{name}",
            )

            if api_call == dict():
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def add_mute_timing(
        self, mute_time_interval: MuteTimeInterval, disable_provenance: bool = False
    ):
        """The method includes a functionality to create a mute timing

        Args:
            mute_time_interval (MuteTimeInterval): Specify the mute time interval
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if mute_time_interval is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/mute-timings",
                RequestsMethods.POST,
                json.dumps(self._create_mute_timing_dictionary(mute_time_interval)),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully added the mute timing.")
        else:
            logging.error("There is no mute_time_interval defined.")
            raise ValueError

    def update_mute_timing(
        self,
        name: str,
        mute_time_interval: MuteTimeInterval,
        disable_provenance: bool = False,
    ):
        """The method includes a functionality to update an existing mute timing

        Args:
            name (str): Specify the mute timing name
            mute_time_interval (MuteTimeInterval): Specify the mute time interval
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(name) != 0 and mute_time_interval is not None:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/mute-timings/{name}",
                RequestsMethods.PUT,
                json.dumps(self._create_mute_timing_dictionary(mute_time_interval)),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully updated the mute timing.")
        else:
            logging.error("There is no name or mute_time_interval defined.")
            raise ValueError

    def delete_mute_timing(self, name: str):
        """The method includes a functionality to delete a mute timings specified by the name

        Args:
            name (str): Specify the mute timing name

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/mute-timings/{name}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the mute timing.")
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def get_all_message_templates(self) -> list:
        """The method includes a functionality to get all message templates

        Raises:
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (list): Returns all message templates
        """

        api_call: list = Api(self.grafana_api_model).call_the_api(
            f"{APIEndpoints.ALERTING_PROVISIONING.value}/templates",
        )

        if isinstance(api_call, list) is False:
            logging.error(f"Check the error: {api_call}.")
            raise Exception
        else:
            return api_call

    def get_message_template(self, name: str) -> dict:
        """The method includes a functionality to get a message template specified by the name

        Args:
            name (str): Specify the message template name

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            api_call (dict): Returns the message template
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/templates/{name}",
                response_status_code=True,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                return api_call
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def create_or_update_message_template(
        self, name: str, message_template: str, disable_provenance: bool = False
    ):
        """The method includes a functionality to create or update a message template

        Args:
            name (str): Specify the message template name
            message_template (str): Specify the message template
            disable_provenance (bool): Specify if the provenance header should be set or not (default False)

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(name) != 0 and len(message_template) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/templates/{name}",
                RequestsMethods.PUT,
                json.dumps({"template": message_template}),
                response_status_code=True,
                disable_provenance_header=disable_provenance,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully created/ updated the message template.")
        else:
            logging.error("There is no name or message defined.")
            raise ValueError

    def delete_message_template(self, name: str):
        """The method includes a functionality to delete a message template

        Args:
            name (str): Specify the message template name

        Raises:
            ValueError: Missed specifying a necessary value
            Exception: Unspecified error by executing the API call

        Returns:
            None
        """

        if len(name) != 0:
            api_call: dict = Api(self.grafana_api_model).call_the_api(
                f"{APIEndpoints.ALERTING_PROVISIONING.value}/templates/{name}",
                RequestsMethods.DELETE,
                response_status_code=True,
            )

            if 200 <= api_call.get("status") >= 300:
                logging.error(f"Check the error: {api_call}.")
                raise Exception
            else:
                logging.info("You successfully deleted the message template.")
        else:
            logging.error("There is no name defined.")
            raise ValueError

    def _create_mute_timing_dictionary(
        self, mute_time_interval: MuteTimeInterval
    ) -> dict:
        """The method includes a functionality to create the mute timing dictionary

        Args:
            mute_time_interval (MuteTimeInterval): Specify the mute time interval

        Returns:
            result (dict): Returns the mute timing dictionary
        """

        return dict(
            {
                "name": mute_time_interval.name,
                "time_intervals": self._create_mute_timing_interval_list(
                    mute_time_interval.time_intervals
                ),
            }
        )

    def _create_mute_timing_interval_list(
        self, time_intervals: List[TimeInterval]
    ) -> (list, None):
        """The method includes a functionality to create the mute timing interval list

        Args:
            time_intervals (List[TimeInterval]): Specify the list of time intervals

        Returns:
            result (list, None): Returns the mute time interval list or None
        """

        mute_timing_interval_list: list = list()

        if time_intervals is not None and isinstance(time_intervals, list):
            for time_interval in time_intervals:
                mute_timing_interval_list.append(
                    {
                        "days_of_month": time_interval.days_of_month,
                        "months": time_interval.months,
                        "times": self._create_time_range_list(time_interval.times),
                        "weekdays": time_interval.weekdays,
                        "years": time_interval.years,
                    }
                )

            return mute_timing_interval_list
        else:
            return time_intervals

    @staticmethod
    def _create_time_range_list(timing: List[TimeRange]) -> (list, None):
        """The method includes a functionality to create the time range list

        Args:
            timing (List[TimeRange]): Specify the list of time points

        Returns:
            timing_list (list): Returns the time list
        """

        timing_list: list = list()

        if timing is not None and isinstance(timing, list):
            for time in timing:
                timing_list.append(
                    {"start_time": time.start_time, "end_time": time.end_time}
                )

            return timing_list
        else:
            return timing

    def _create_alert_route_dictionary(self, route: Route) -> dict:
        """The method includes a functionality to create the alert route dictionary

        Args:
            route (Route): Specify the route

        Returns:
            result (dict): Returns the alert route dictionary
        """

        return dict(
            {
                "continue": route.continue_parameter,
                "group_by": route.group_by_str,
                "mute_time_intervals": route.mute_time_intervals,
                "receiver": route.receiver,
                "routes": self._create_alert_routes_list(route.routes),
                "group_interval": route.group_interval,
                "group_wait": route.group_wait,
                "object_matchers": self._create_object_matcher_list(
                    route.object_matchers
                ),
                "provenance": route.provenance,
                "repeat_interval": route.repeat_interval,
            }
        )

    def _create_alert_routes_list(self, routes: List[Route]) -> (list, None):
        """The method includes a functionality to create the alert route list

        Args:
            routes (List[Route]): Specify the list of alert routes

        Returns:
            result (list, None): Returns the alert routes list or None
        """

        routes_list: list = list()

        if routes is not None and isinstance(routes, list):
            for route in routes:
                routes_list.append(self._create_alert_route_dictionary(route))

            return routes_list
        else:
            return routes

    @staticmethod
    def _create_object_matcher_list(matchers: List[Matcher]) -> (list, None):
        """The method includes a functionality to create the object matcher list

        Args:
            matchers (List[Matcher]): Specify the list of object matchers

        Returns:
            route_matchers_list (list): Returns the list of object matchers
        """

        route_matchers_list: list = list()

        if matchers is not None and isinstance(matchers, list):
            for matcher in matchers:
                route_matcher_dict: dict = dict(
                    {
                        "name": matcher.name,
                        "type": matcher.type.value,
                        "value": matcher.value,
                    }
                )

                route_matchers_list.append(route_matcher_dict)

            return route_matchers_list
        else:
            return matchers

    def _create_alert_rule_dictionary(self, alert_rule: AlertRule) -> dict:
        """The method includes a functionality to create the alert rule dictionary

        Args:
            alert_rule (AlertRule): Specify the alert rule

        Returns:
            result (dict): Returns the alert rule dictionary
        """

        return dict(
            {
                "annotations": alert_rule.annotations,
                "condition": alert_rule.condition,
                "data": self._create_alert_rule_query_list(alert_rule.data),
                "execErrState": alert_rule.exec_err_state,
                "folderUID": alert_rule.folder_uid,
                "id": alert_rule.id,
                "labels": alert_rule.labels,
                "noDataState": alert_rule.no_data_state,
                "orgID": alert_rule.org_id,
                "ruleGroup": alert_rule.rule_group,
                "title": alert_rule.title,
                "uid": alert_rule.uid,
                "updated": alert_rule.updated,
                "for": alert_rule.for_time,
                "provenance": alert_rule.provenance,
            }
        )

    def _create_alert_rule_query_list(self, alert_queries: List[AlertQuery]) -> list:
        """The method includes a functionality to create the alert rule query list

        Args:
            alert_queries (List[AlertQuery]): Specify the alert rule query list

        Returns:
            alert_rule_queries_list (list): Returns the alert rule query list
        """

        alert_rule_queries_list: list = list()

        for alert_query in alert_queries:
            alert_query_dict: dict = dict(
                {
                    "datasourceUID": alert_query.datasource_uid,
                    "model": self._create_alert_rule_query_model_dictionary(
                        alert_query.model
                    ),
                    "queryType": alert_query.query_type,
                    "refID": alert_query.ref_id,
                    "relativeTimeRange": {
                        "from": alert_query.relative_time_range_from,
                        "to": alert_query.relative_time_range_to,
                    },
                }
            )

            alert_rule_queries_list.append(alert_query_dict)

        return alert_rule_queries_list

    def _create_alert_rule_query_model_dictionary(
        self, alert_query_model: AlertRuleQueryModel
    ) -> dict:
        """The method includes a functionality to create the alert rule query model dictionary

        Args:
            alert_query_model (AlertRuleQueryModel): Specify the alert rule query model

        Returns:
            result (dict): Returns the alert rule query model dictionary
        """

        return dict(
            {
                "conditions": self._create_alert_rule_query_model_condition_list(
                    alert_query_model.conditions
                ),
                "datasource": alert_query_model.datasource,
                "expression": alert_query_model.expression,
                "hide": alert_query_model.hide,
                "intervalMs": alert_query_model.interval_ms,
                "maxDataPoints": alert_query_model.max_data_points,
                "refId": alert_query_model.ref_id,
                "type": alert_query_model.type,
            }
        )

    @staticmethod
    def _create_alert_rule_query_model_condition_list(
        alert_rule_query_model_conditions: List[AlertRuleQueryModelCondition],
    ) -> list:
        """The method includes a functionality to create the alert rule query model condition list

        Args:
            alert_rule_query_model_conditions (List[AlertRuleQueryModelCondition]): Specify the alert rule query model conditions list

        Returns:
            alert_rule_query_model_conditions_list (list): Returns the alert rule query model conditions list
        """

        alert_rule_query_model_conditions_list: list = list()

        for alert_rule_query_model_condition in alert_rule_query_model_conditions:
            alert_rule_query_model_condition_dict: dict = dict(
                {
                    "evaluator": {
                        "params": alert_rule_query_model_condition.evaluator_params,
                        "type": alert_rule_query_model_condition.evaluator_type,
                    },
                    "operator": {
                        "type": alert_rule_query_model_condition.operator_type
                    },
                    "query": {"params": alert_rule_query_model_condition.query_params},
                    "reducer": {
                        "params": alert_rule_query_model_condition.reducer_params,
                        "type": alert_rule_query_model_condition.reducer_type,
                    },
                    "type": alert_rule_query_model_condition.type,
                }
            )

            alert_rule_query_model_conditions_list.append(
                alert_rule_query_model_condition_dict
            )

        return alert_rule_query_model_conditions_list
